/* arch/arm/mach-s3c2410/s3c2443-bugs.c
 *
 * Workarounds for various bugs in the Samsung S3C2443 CPU.
 *
 * Copyright (C) 2007 TomTom BV <http://www.tomtom.com/>
 * Author: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifdef __KERNEL__
#include <asm/io.h>
#include <asm/system.h>
#include <asm/arch/regs-gpio.h>
#include "s3c2443-bugs.h"

unsigned s3c2443_read_gpacdl(void)
{
	unsigned long flags;
	unsigned res;

	local_irq_save(flags);
	res = s3c2443_convert_2_gpacdl(__raw_readl(S3C2410_GPACON), __raw_readl(S3C2410_GPADAT));
	local_irq_restore(flags);
	return res;
}

unsigned s3c2443_read_gpacdh(void)
{
	unsigned long flags;
	unsigned res;

	local_irq_save(flags);
	res = s3c2443_convert_2_gpacdh(__raw_readl(S3C2410_GPACON), __raw_readl(S3C2410_GPADAT));
	local_irq_restore(flags);
	return res;
}

unsigned s3c2443_read_extint0(void)
{
	unsigned reg;

	reg = __raw_readl(S3C2410_EXTINT0);
	return s3c2443_convert_extint0(reg);
}

unsigned s3c2443_read_extint1(void)
{
	unsigned reg;

	reg = __raw_readl(S3C2410_EXTINT1);
	return s3c2443_convert_extint12(reg);
}

unsigned s3c2443_read_extint2(void)
{
	unsigned reg;

	reg = __raw_readl(S3C2410_EXTINT2);
	return s3c2443_convert_extint12(reg);
}

unsigned s3c2443_read_einflt2(void)
{
	unsigned reg;

	reg = __raw_readl(S3C2410_EINFLT2);
	reg = (((reg & 0xfe000000) >> 25) |	/* 1111 1110 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x01000000) >> 17) |	/* 0000 0001 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x00fe0000) >>  9) |	/* 0000 0000 1111 1110 0000 0000 0000 0000 */
	       ((reg & 0x00010000) >>  1) |	/* 0000 0000 0000 0001 0000 0000 0000 0000 */
	       ((reg & 0x0000fe00) <<  7) |	/* 0000 0000 0000 0000 1111 1110 0000 0000 */
	       ((reg & 0x00000100) << 15) |	/* 0000 0000 0000 0000 0000 0001 0000 0000 */
	       ((reg & 0x000000fe) << 23) |	/* 0000 0000 0000 0000 0000 0000 1111 1110 */
	       ((reg & 0x00000001) << 31));	/* 0000 0000 0000 0000 0000 0000 0000 0001 */

	return reg;
}

unsigned s3c2443_read_einflt3(void)
{
	unsigned reg;

	reg = __raw_readl(S3C2410_EINFLT3);
	reg = (((reg & 0xfe000000) >> 25) |	/* 1111 1110 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x01000000) >> 17) |	/* 0000 0001 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x00fe0000) >>  9) |	/* 0000 0000 1111 1110 0000 0000 0000 0000 */
	       ((reg & 0x00010000) >>  1) |	/* 0000 0000 0000 0001 0000 0000 0000 0000 */
	       ((reg & 0x0000fe00) <<  7) |	/* 0000 0000 0000 0000 1111 1110 0000 0000 */
	       ((reg & 0x00000100) << 15) |	/* 0000 0000 0000 0000 0000 0001 0000 0000 */
	       ((reg & 0x000000fe) << 23) |	/* 0000 0000 0000 0000 0000 0000 1111 1110 */
	       ((reg & 0x00000001) << 31));	/* 0000 0000 0000 0000 0000 0000 0000 0001 */

	return reg;
}
#endif /* __KERNEL__ */

unsigned s3c2443_convert_2_gpacdl(unsigned gpacdl, unsigned gpacdh)
{
	unsigned res;
	
	gpacdl &= 0xff;
	gpacdh &= 0xff;
	res = ((((gpacdh >> 0) & 1) <<  0) |
	       (((gpacdl >> 0) & 1) <<  1) |
	       (((gpacdh >> 1) & 1) <<  2) |
	       (((gpacdl >> 1) & 1) <<  3) |
	       (((gpacdh >> 2) & 1) <<  4) |
	       (((gpacdl >> 2) & 1) <<  5) |
	       (((gpacdh >> 3) & 1) <<  6) |
	       (((gpacdl >> 3) & 1) <<  7) |
	       (((gpacdh >> 4) & 1) <<  8) |
	       (((gpacdl >> 4) & 1) <<  9) |
	       (((gpacdh >> 5) & 1) << 10) |
	       (((gpacdl >> 5) & 1) << 11) |
	       (((gpacdh >> 6) & 1) << 12) |
	       (((gpacdl >> 6) & 1) << 13) |
	       (((gpacdh >> 7) & 1) << 14) |
	       (((gpacdl >> 7) & 1) << 15));

	return res;
}

unsigned s3c2443_convert_2_gpacdh(unsigned gpacdl, unsigned gpacdh)
{
	unsigned res;

	gpacdl = gpacdl >> 8;
	gpacdh = gpacdh >> 8;
	res = ((((gpacdh >> 0) & 1) <<  0) |
	       (((gpacdl >> 0) & 1) <<  1) |
	       (((gpacdh >> 1) & 1) <<  2) |
	       (((gpacdl >> 1) & 1) <<  3) |
	       (((gpacdh >> 2) & 1) <<  4) |
	       (((gpacdl >> 2) & 1) <<  5) |
	       (((gpacdh >> 3) & 1) <<  6) |
	       (((gpacdl >> 3) & 1) <<  7) |
	       (((gpacdh >> 4) & 1) <<  8) |
	       (((gpacdl >> 4) & 1) <<  9) |
	       (((gpacdh >> 5) & 1) << 10) |
	       (((gpacdl >> 5) & 1) << 11) |
	       (((gpacdh >> 6) & 1) << 12) |
	       (((gpacdl >> 6) & 1) << 13) |
	       (((gpacdh >> 7) & 1) << 14) |
	       (((gpacdl >> 7) & 1) << 15) |
	       (((gpacdh >> 8) & 1) << 16));

	return res;
} 
unsigned s3c2443_convert_extint0(unsigned reg)
{
	reg = (((reg & 0xf0000000) >> 28) |	/* 1111 0000 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x0f000000) >> 20) |	/* 0000 1111 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x00f00000) >> 12) |	/* 0000 0000 1111 0000 0000 0000 0000 0000 */
	       ((reg & 0x000f0000) >>  4) |	/* 0000 0000 0000 1111 0000 0000 0000 0000 */
	       ((reg & 0x0000f000) <<  4) |	/* 0000 0000 0000 0000 1111 0000 0000 0000 */
	       ((reg & 0x00000f00) << 12) |	/* 0000 0000 0000 0000 0000 1111 0000 0000 */
	       ((reg & 0x000000f0) << 20) |	/* 0000 0000 0000 0000 0000 0000 1111 0000 */
	       ((reg & 0x0000000f) << 28));	/* 0000 0000 0000 0000 0000 0000 0000 1111 */
       
	return reg;
}

unsigned s3c2443_convert_extint12(unsigned reg)
{
	reg = (((reg & 0xe0000000) >> 29) |	/* 1110 0000 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x10000000) >> 25) |	/* 0001 0000 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x0e000000) >> 21) |	/* 0000 1110 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x01000000) >> 17) |	/* 0000 0001 0000 0000 0000 0000 0000 0000 */
	       ((reg & 0x00e00000) >> 13) |	/* 0000 0000 1110 0000 0000 0000 0000 0000 */
	       ((reg & 0x00100000) >>  9) |	/* 0000 0000 0001 0000 0000 0000 0000 0000 */
	       ((reg & 0x000e0000) >>  5) |	/* 0000 0000 0000 1110 0000 0000 0000 0000 */
	       ((reg & 0x00010000) >>  1) |	/* 0000 0000 0000 0001 0000 0000 0000 0000 */
	       ((reg & 0x0000e000) <<  3) |	/* 0000 0000 0000 0000 1110 0000 0000 0000 */
	       ((reg & 0x00001000) <<  7) |	/* 0000 0000 0000 0000 0001 0000 0000 0000 */
	       ((reg & 0x00000e00) << 11) |	/* 0000 0000 0000 0000 0000 1110 0000 0000 */
	       ((reg & 0x00000100) << 15) |	/* 0000 0000 0000 0000 0000 0001 0000 0000 */
	       ((reg & 0x000000e0) << 19) |	/* 0000 0000 0000 0000 0000 0000 1110 0000 */
	       ((reg & 0x00000010) << 23) |	/* 0000 0000 0000 0000 0000 0000 0001 0000 */
	       ((reg & 0x0000000e) << 27) |	/* 0000 0000 0000 0000 0000 0000 0000 1110 */
	       ((reg & 0x00000001) << 31));	/* 0000 0000 0000 0000 0000 0000 0000 0001 */

	return reg;
}

/* EOF */

