/* drivers/barcelona/powerhold/powerhold.c
*
* Implementation of the PowerHold driver.
*
* Copyright (C) 2008 TomTom BV <http://www.tomtom.com/>
* Author  Iwan Sanders <Iwan.Sanders@tomtom.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*/

/* Includes */ 
#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/wait.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/poll.h>
#include <linux/device.h>
#include <linux/interrupt.h>
#include <linux/proc_fs.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/hardware.h>
#include <asm/hardware/clock.h>
#include <asm/delay.h>
#include <barcelona/Barc_micro.h>
#include <barcelona/debug.h>
#include <barcelona/gopins.h>
#include <asm/uaccess.h>	/* for copy_from_user */

// Defines
#define PFX "powerhold: "
#define PK_DBG PK_DBG_FUNC
#define PROCFS_NAME     "barcelona/powerhold"
#define PROCFS_MAX_SIZE	50

struct proc_dir_entry *Our_Proc_File;
static char procfs_buffer[PROCFS_MAX_SIZE];
static unsigned long procfs_buffer_size = 0;


void check_powerhold_state(void)
{
  if ( IO_GetInput( PWR_HOLD ) )
  {
  	procfs_buffer_size = sprintf(procfs_buffer,"enabled\n");
  }
  else
  {
  	procfs_buffer_size = sprintf(procfs_buffer,"disabled\n");
  }
}

int
procfile_read(char *buffer,
	      char **buffer_location,
	      off_t offset, int buffer_length, int *eof, void *data)
{
	int ret;
	/* Retreiving actual state of the POWER_HLD pin. */
	check_powerhold_state();
	if (offset > 0) {
		/* we have finished to read, return 0 */
		ret  = 0;
	} else {
		/* fill the buffer, return the buffer size */
		memcpy(buffer, procfs_buffer, procfs_buffer_size);
		ret = procfs_buffer_size; 
	}

	return ret;
}

int procfile_write(struct file *file, const char *buffer, unsigned long count,
		   void *data)
{
	
	/* write data to the buffer */
	if( strstr(buffer,"enabled")!= 0 || strstr(buffer,"disabled")!=0)
	{
		/* get buffer size */
		procfs_buffer_size = count;
		if (procfs_buffer_size > PROCFS_MAX_SIZE ) {
			procfs_buffer_size = PROCFS_MAX_SIZE;
		}
		
		if( strncmp(buffer, "enabled",7) == 0) 
		{
			IO_Activate(PWR_HOLD);
		}
		else 
		{
			IO_Deactivate( PWR_HOLD );
		}
		
		if ( copy_from_user(procfs_buffer, buffer, procfs_buffer_size) ) 
		{
			return -EFAULT;
		}
	}
	else 
	{
		printk(KERN_ALERT "PowerHold Driver Error: only enabled or disabled is supported!\n");
	}
	return procfs_buffer_size;
}



static int powerhold_probe(struct device *dev)
{
  return 0;
}

static int powerhold_remove(struct device *dev)
{
  return 0;
}

static void powerhold_shutdown(struct device *dev)
{
  PK_DBG("Shutting down PowerHold\n");
}

static struct device_driver powerhold_driver = {
  .name		  	= "tomtomgo-powerhold",
  .bus	    		= &platform_bus_type,
  .probe		= powerhold_probe,
  .remove		= powerhold_remove,
  .shutdown		= powerhold_shutdown,
};

static int __init powerhold_mod_init(void)
{
  int ret;

  printk(KERN_INFO "TomTom GO PowerHold Driver, (C) 2008 TomTom BV\n");

  PK_DBG("Registering driver\n");
  ret = driver_register(&powerhold_driver);
  if (ret) {
    PK_ERR("Unable to register driver (%d)\n", ret);
    return ret;
  }
  
  Our_Proc_File = create_proc_entry(PROCFS_NAME, 0644, NULL);
  if (Our_Proc_File == NULL) {
  	remove_proc_entry(PROCFS_NAME, &proc_root);
	printk(KERN_ALERT "Error: Could not initialize /proc/%s\n",
		PROCFS_NAME);
	return -ENOMEM;
  }
  
  Our_Proc_File->read_proc 	= procfile_read;
  Our_Proc_File->write_proc	= procfile_write;
  Our_Proc_File->owner 	 	= THIS_MODULE;
  Our_Proc_File->mode 	 	= S_IFREG | S_IRUGO;
  Our_Proc_File->uid 	 	= 0;
  Our_Proc_File->gid 	 	= 0;
  Our_Proc_File->size 	 	= 37;
  check_powerhold_state();	  
  return ret;
}

static void __exit powerhold_mod_exit(void)
{
  PK_DBG("Unregistering driver\n");
  driver_unregister(&powerhold_driver);
  remove_proc_entry(PROCFS_NAME, &proc_root);
}

module_init(powerhold_mod_init);
module_exit(powerhold_mod_exit);

MODULE_AUTHOR("Iwan Sanders <Iwan.Sanders@tomtom.com>");
MODULE_DESCRIPTION("TomTom PowerHold Driver");
MODULE_LICENSE("GPL");

/* EOF */ 
