/* drivers/barcelona/usbmode/powermon.c
Copyright (C) 2007 TomTom BV <http://www.tomtom.com/>
Author: Kees Jongenburger <kees.jongenburger@tomtom.com>

Functions in this file preform sub power monitoring.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2 as
published by the Free Software Foundation.
*/
#include <linux/kernel.h>

#include <linux/delay.h>
#include <linux/notifier.h>
#include <barcelona/gotype.h>
#include <barcelona/gopins.h>
#include <barcelona/usbmode.h>
#include <barcelona/Barc_buspower.h>
#include <asm/io.h>
#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-clock.h>
#include <asm/arch/regs-hsudc.h>
#include <asm/arch/regs-udc.h>
#include <asm/arch/regs-dyn.h>
#include <asm/arch/regs-irq.h>

/* Interval we will check if there is buspower on the device */
#define VBUS_PROBE_INTERVAL 2

/* Defines */
#define PFX "usbmode:powermon"
#define ERR " ERROR"
#define PK_DBG PK_DBG_FUNC

/* Function used when we use poll based USB_HOST_DETECT monitoring */
int buspower_changed(struct notifier_block *self, unsigned long power_on,
		     void *data_pointer)
{
	struct usbmode_data *data =
	    (struct usbmode_data *)container_of(self, struct usbmode_data,
						buspower_change_listener);
	unsigned int current_input_state = data->input_state;
	printk(KERN_DEBUG PFX " %s POWER EVENT \n", __FUNCTION__);
	if (power_on) {
		if (data->input_state & INPUT_BUS_POWER_OFF_EVENT) {
			data->input_state &= ~INPUT_BUS_POWER_OFF_EVENT;
		}
		data->input_state |= INPUT_BUS_POWER_ON_EVENT;
	} else {
		if (data->input_state & INPUT_BUS_POWER_ON_EVENT) {
			data->input_state &= ~INPUT_BUS_POWER_ON_EVENT;
		}
		data->input_state |= INPUT_BUS_POWER_OFF_EVENT;
	}

	/* if we have an input state change */
	if (current_input_state != data->input_state) {
		schedule_work(&data->state_changed_work);
	}
	return 0;
}

/* start the power monitor */
void start_powermon(struct usbmode_data *data)
{
	data->buspower_change_listener.notifier_call = buspower_changed;
	buspower_register_notifier(&data->buspower_change_listener);

	/* the rest of the code here is to handle the situation when there is already power
	   on the bus when we start the power monitor */
	unsigned int current_input_state = data->input_state;
	if (buspower_get_status()) {	//if we have power send an event
		if (data->input_state & INPUT_BUS_POWER_OFF_EVENT) {
			data->input_state &= ~INPUT_BUS_POWER_OFF_EVENT;
		}
		data->input_state |= INPUT_BUS_POWER_ON_EVENT;
	}
	if (current_input_state != data->input_state) {
		schedule_work(&data->state_changed_work);
	}
};

/* stop the power monitor */
void stop_powermon(struct usbmode_data *data)
{
	buspower_unregister_notifier(&data->buspower_change_listener);
};
