/* drivers/barcelona/usbmode/usbmodechange.c
Copyright (C) 2007 TomTom BV <http://www.tomtom.com/>
Author: Kees Jongenburger <kees.jongenburger@tomtom.com>

Methods to perform state changes between the different usb modes

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2 as
published by the Free Software Foundation.
*/
#include <linux/kernel.h>
#include <barcelona/usbmode.h>
#include "usbmodechange.h"

#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/hardware/clock.h>
#include <asm/io.h>
#include <linux/kernel.h>
#include <linux/err.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/interrupt.h>
#include <linux/usb.h>
#include <linux/workqueue.h>

#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-clock.h>
#include <asm/arch/regs-hsudc.h>
#include <asm/arch/regs-udc.h>
#include <asm/arch/regs-dyn.h>

#include <barcelona/gotype.h>
#include <barcelona/gopins.h>

/* Defines */
#define PFX "usbmode:usbmodechange:"
#define ERR " ERROR"
#define PK_DBG PK_DBG_FUNC

void helper_enable_host_port(struct usbmode_data *data)
{
	unsigned int i;
	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:
		/* route USB PHY DP/DN pins to host controller */
		i = __raw_readl(S3C2443_PHYCTRL);
		i |= S3C2443_PHYCTRL_DOWNSTREAM_PORT;
		__raw_writel(i, S3C2443_PHYCTRL);
		break;
	case GOCPU_S3C2410:
	case GOCPU_S3C2412:
	case GOCPU_S3C2440:
	case GOCPU_S3C2442:
		i = __raw_readl(s3c24xx_misccr);
		i |= S3C2412_MISCCR_USBHOST;	// (1<<3);
		i &= ~S3C2412_MISCCR_USBSUSPND1;	// (1<<13);
		i &= ~S3C2412_MISCCR_USBSUSPND0;	// (1<<12);
		if (IO_UsbOhciPortMask() & 0x01) {
			i |= S3C2412_MISCCR_USBSUSPND0;	// (1<<12);
		}
		if (IO_UsbOhciPortMask() & 0x02) {
			i |= S3C2412_MISCCR_USBSUSPND1;	// (1<<13);
		}
		__raw_writel(i, s3c24xx_misccr);
		break;
	default:
		printk(KERN_INFO PFX ERR
		       " %s Does not know how to enable host port on this platform\n",
		       __FUNCTION__);
	}
}

void helper_disable_host_port(struct usbmode_data *data)
{
	unsigned int i;
	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:
		/* route USB PHY DP/DN pins to host controller */
		i = __raw_readl(S3C2443_PHYCTRL);
		i &= ~(S3C2443_PHYCTRL_DOWNSTREAM_PORT);
		__raw_writel(i, S3C2443_PHYCTRL);
		break;
	case GOCPU_S3C2410:
	case GOCPU_S3C2412:
	case GOCPU_S3C2440:
	case GOCPU_S3C2442:
		i = __raw_readl(s3c24xx_misccr);
		i &= ~(S3C2412_MISCCR_USBHOST | S3C2412_MISCCR_USBSUSPND1);
		i |= S3C2412_MISCCR_USBSUSPND0;	// (1<<3) |(1<<13) ;
		__raw_writel(i, s3c24xx_misccr);
		break;
	default:
		printk(KERN_INFO PFX ERR
		       " %s Does not know how to disable host port on this platform\n",
		       __FUNCTION__);
	}

}

void helper_suspend_usb(void) 
{
	unsigned int i;
	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:

		/* Force suspend usb block */
		i = __raw_readl(S3C2443_PHYPWR);
		i |= S3C2443_PHYPWR_FORCE_SUSPEND;
		__raw_writel(i, S3C2443_PHYPWR);

		break;

	case GOCPU_S3C2410:
	case GOCPU_S3C2412:
	case GOCPU_S3C2440:
	case GOCPU_S3C2442:

		i = __raw_readl(s3c24xx_misccr);
		i &= ~(S3C2412_MISCCR_USBHOST | S3C2412_MISCCR_USBSUSPND1);
		i |= S3C2412_MISCCR_USBSUSPND0;	// (1<<3) |(1<<13) ;
		__raw_writel(i, s3c24xx_misccr);

		break;
	default:
		printk(KERN_INFO PFX ERR
		       " %s Does not know how to suspend usb on this platform\n",
		       __FUNCTION__);
	}
	
	mdelay(500);
}

/* Start usb device detect mode . in this mode we try to detect if we must behave like a usb device */
void device_detect_timeout(unsigned long data_pointer)
{
	struct usbmode_data *data = (struct usbmode_data *)data_pointer;
	if (data->usb_state == USB_STATE_DEVICE_DETECT) {
		printk(KERN_DEBUG PFX " %s timeout\n", __FUNCTION__);
		data->input_state |= INPUT_DEVICE_DETECT_TIMEOUT;
		schedule_work(&data->state_changed_work);
	} else {
		printk(KERN_DEBUG PFX ERR " %s timeout\n", __FUNCTION__);
	}
}

static irqreturn_t
device_detect_irq_handler(int irq, void *data_pointer, struct pt_regs *regs)
{
	struct usbmode_data *data = (struct usbmode_data *)data_pointer;
	int handled;
	int i;

	do {
		int saveIdx = __raw_readl(S3C_UDC_INDEX_REG);
		int usb_status = __raw_readl(S3C_UDC_USB_INT_REG);
		int usbd_status = __raw_readl(S3C_UDC_EP_INT_REG);

		handled = 0;

		/* RESET Interrupt Request - USB reset */
		if (usb_status & S3C_UDC_USBINT_RESET) {
			/* clear interrupt */
			__raw_writel(S3C_UDC_USBINT_RESET, S3C_UDC_USB_INT_REG);

			/* we have the signal we where looking for (a reset) */
			/* we now change the input_state bit and fire the state change tasklet */
			data->input_state |= INPUT_DEVICE_RESET;
			printk(KERN_DEBUG PFX " RESET\n");
			schedule_work(&data->state_changed_work);
			handled = 1;
		}

		/* RESume Interrupt Request */
		if (usb_status & S3C_UDC_USBINT_RESUME) {
			/* clear interrupt */
			printk(KERN_DEBUG PFX " RESUME\n");
			__raw_writel(S3C_UDC_USBINT_RESUME,
				     S3C_UDC_USB_INT_REG);
			handled = 1;
		}

		/* SUSpend Interrupt Request */
		if (usb_status & S3C_UDC_USBINT_SUSPEND) {
			/* clear interrupt */
			printk(KERN_DEBUG PFX " SUSPEND\n");
			__raw_writel(S3C_UDC_USBINT_SUSPEND,
				     S3C_UDC_USB_INT_REG);
			handled = 1;
		}

		/* EP interrupts */
		/*while we did not register for EP interrupts we still get EP0 interrupts */
		if (usbd_status) {
			/* endpoint data transfers */
			for (i = 0; i < 5; i++) {
				u32 tmp = 1 << i;
				if (usbd_status & tmp) {
					/* Clear the interrupt bit by setting it to 1 */
					__raw_writel(tmp, S3C_UDC_EP_INT_REG);
					handled = 1;
				}
			}
			handled = 1;
		}
		/* restore index reg */
		__raw_writel(saveIdx, S3C_UDC_INDEX_REG);
	} while (handled);

	return IRQ_HANDLED;
}

static irqreturn_t usb2_irq_handler(int irq, void *data_pointer,
				    struct pt_regs *regs)
{
	struct usbmode_data *data = (struct usbmode_data *)data_pointer;

	unsigned int s, ep_int, tr;
	unsigned int fnr;

	/* Get interrupt source */
	s = __raw_readl(S3C_HSUDC_SYS_STATUS_REG);
	ep_int = __raw_readl(S3C_HSUDC_EP_INT_REG);
	tr = __raw_readl(S3C_HSUDC_TEST_REG);
	fnr = __raw_readl(S3C_HSUDC_FRAME_NUM_REG);

	/* Ack any endpoint activity */
	if (ep_int) {
		__raw_writel(ep_int, S3C_HSUDC_EP_INT_REG);
	}

	if (s) {
		if (s & S3C_HSUDC_INT_RESET) {
			/* we have the signal we where looking for (a reset) */
			/* we now change the input_state bit and fire the state change tasklet */

			/*TODO perhaps also enable the device reset counter here just like for the
			   non 2442 irq handler */
			data->input_state |= INPUT_DEVICE_RESET;
			schedule_work(&data->state_changed_work);
		}
		if (s & S3C_HSUDC_INT_VBUSON) {
		}
		if (s & S3C_HSUDC_INT_VBUSOFF) {
		}

		__raw_writel(s, S3C_HSUDC_SYS_STATUS_REG);
	}

	if (tr) {
		__raw_writel(tr, S3C_HSUDC_TEST_REG);
	}

	return IRQ_HANDLED;
}

/* check the registers to fix any issue with the clocks */
int setup_clocks_for_device(struct usbmode_data *data)
{
	unsigned long i;

	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:

		if (__raw_readl(S3C2443_RSTSTAT) & S3C2443_RSTSTAT_SLEEP) {
			__raw_writel(__raw_readl(S3C2443_RSTCON) |
				     S3C2443_RSTCON_PWROFF_SLEEP,
				     S3C2443_RSTCON);
		}
		if (IO_UsbOhciPortMask() & 0x01) {
			/* put USB port in normal mode (not suspend) */
			__raw_writel(__raw_readl(s3c24xx_misccr) &
				     ~(S3C2412_MISCCR_USBSUSPND0),
				     s3c24xx_misccr);
		} else {

			__raw_writel(__raw_readl(s3c24xx_misccr) &
				     ~(S3C2412_MISCCR_USBSUSPND1),
				     s3c24xx_misccr);
		}

		/* enable external USB PHY power regulator */
		IO_Activate(USB_PHY_PWR_EN);
		mdelay(1);

		__raw_writel(__raw_readl(S3C2443_PWRCFG) |
			     S3C2443_PWRCFG_nSW_PHY_OFF_USB, S3C2443_PWRCFG);

		/* USB device 2.0 must reset like below,
		   1st phy reset and after at least 10us, func_reset & host reset
		   phy reset can reset below registers.
		 */

		__raw_writel(S3C2443_URSTCON_PHY_RESET, S3C2443_URSTCON);
		udelay(20);	/* phy reset must be asserted for at least 10us */

		/*Function 2.0, Host 1.1 S/W reset, deassert PHY reset */
		//__raw_writel( S3C2443_URSTCON_FUNC_RESET | S3C2443_URSTCON_HOST_RESET, S3C2443_URSTCON);
		__raw_writel(S3C2443_URSTCON_FUNC_RESET, S3C2443_URSTCON);

		udelay(20);
		__raw_writel(0, S3C2443_URSTCON);	/* deassert all resets  */

		mdelay(10);

		/* 12Mhz clock on ,PHY2.0 analog block power on
		   XO block power on,XO block power in suspend mode,
		   PHY 2.0 Pll power on ,suspend signal for save mode disable
		 */
		__raw_writel(S3C2443_PHYPWR_COMMON_ON_N, S3C2443_PHYPWR);

		/* D+ pull up disable(VBUS detect), USB2.0 Function clock Enable,
		   USB1.1 HOST disable, USB2.0 PHY test disable */
		__raw_writel(S3C2443_UCLKCON_DETECT_VBUS |	/* detect VBUS */
			     S3C2443_UCLKCON_HOST_CLK_TEST |	/* DISable HOST_CLK_TEST */
			     S3C2443_UCLKCON_FUNC_CLK_EN |	/* USB 2 clock enable */
			     S3C2443_UCLKCON_HOST_CLK_EN, S3C2443_UCLKCON);

		/* error interrupt enable, 16bit bus, Little format,
		   suspend&reset enable
		 */
		__raw_writew(S3C_HSUDC_DTZIEN_EN | S3C_HSUDC_RRD_EN |	/* low byte first NOT supported...      */
			     S3C_HSUDC_SUS_EN | S3C_HSUDC_RST_EN,	/* | S3C2443_UDC_SCR_BIS;       */
			     S3C_HSUDC_SYS_CON_REG);

		__raw_writel(0x00000, S3C_HSUDC_EP0_CON_REG);

		break;
	case GOCPU_S3C2410:
	case GOCPU_S3C2440:
	case GOCPU_S3C2442:
		/* 2410/2440/2442, not 2412 */
		i = __raw_readb(S3C2410_CLKSLOW);
		/* enable usb the documentation points out that it is
		   required to disable CLKSLOW in order the allow the pll
		   clock */
		i &= ~S3C2410_CLKSLOW_USB_CLK_DISABLE;
		__raw_writeb(i, S3C2410_CLKSLOW);
		/* intentional fallthrough */
		break;		/* kejo:todo: is this break needed? is so remove comment above :p */
	case GOCPU_S3C2412:
		/* tomtomgo_init configures the upll clock to 24 Mhz and
		   use a different divider in this code we reprogram the
		   clocks for the 4112. buspower.c also  changed the
		   frequency and settings of the usb host/device mode.
		   we have tried to keep the changes related to usb host to
		   this file, and thus we override both the buspower and
		   tomtomgo_init settings */

		/* we never use the EXTCLK but we use a crystal as source
		   clock that crystal is 12 Mhz */

		/* configure the clock source for the u(sb)pll */
		i = __raw_readl(S3C2412_CLKSRC);
		i &= ~(S3C2412_CLKSRC_SELUREF_MASK);	/* clean up the upll clock source mask( 3 << 12) */
		i |= S3C2412_CLKSRC_SELUREF_EXTOSC;	/* configure the external crystal to be clock source (2 << 12) */

		/* We now have a 12Mhz clock signal to the upll */
		/* Select the upll output to be the source for the USB system clock (USYSCLK = FOUTupll) */
		/* SELUPLL =1 */
		i |= S3C2412_CLKSRC_SELUPLL;	/* (1 <<5) */

		/* the USYSCLK on it's turn must be used a source for the USBSRCCLK */
		/* SELUSB =0 */
		i &= ~(S3C2412_CLKSRC_SELUSB);	/* 1 < 10 */

		__raw_writel(i, S3C2412_CLKSRC);

		/*configure the usb divider */
		i = __raw_readl(S3C2410_CLKDIVN);
		/* 1 = 48Mhz / 2 | 0 = 48Mhz */
		i &= ~(S3C2412_CLKDIVN_USB48DIV);
		__raw_writel(i, S3C2410_CLKDIVN);

		/* We now need to configure the upll and put it on fire */
		//FOUTupll 48Mhz p = 7 m = 64 (0x40) , s 1
		//[20] =0 (ON)
		//[19:12] = MDIV
		//[9:4] = PDIV
		//[0,1] = SDIV
		i &= ~(S3C2410_PLLCON_MDIVMASK | S3C2410_PLLCON_PDIVMASK |
		       S3C2410_PLLCON_SDIVMASK);
		i |= 0x40 << S3C2410_PLLCON_MDIVSHIFT;
		i |= 7 << S3C2410_PLLCON_PDIVSHIFT;
		i |= 1 << S3C2410_PLLCON_SDIVSHIFT;
		__raw_writel(i, S3C2410_UPLLCON);

		mdelay(1);

		/*enable the clock to usb device */
		i = __raw_readl(S3C2410_CLKCON);
		i &= ~S3C2412_CLKCON_USBH;
		i &= ~S3C2412_CLKCON_USBH48M;
		/* TODO:how do we */
		/* enable the usb device mode */
		i |= S3C2412_CLKCON_USBD;
		i |= S3C2412_CLKCON_USBD48M;

		__raw_writel(i, S3C2410_CLKCON);
		break;
	default:
		printk(KERN_INFO PFX ERR "%s no clocks configured\n",
		       __FUNCTION__);
		break;
	}
	return 0;		/* big success */
}

int setup_clocks_for_host(struct usbmode_data *data)
{
	unsigned long i;
	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:
		i = __raw_readl(S3C2443_PWRCFG);
		if (!(i & S3C2443_PWRCFG_nSW_PHY_OFF_USB)) {
			printk(KERN_INFO PFX ERR
			       " S3C2443_PWRCFG register is not set to power the usb phy block (%08lx)\n",
			       i);
			return 1;
		}

		/* check the different registers */

		i = __raw_readl(S3C2443_PHYPWR);
		if ((i & S3C2443_PHYPWR_ANALOG_POWERDOWN_MASK) ==
		    S3C2443_PHYPWR_ANALOG_POWERDOWN_DOWN) {
			printk(KERN_INFO PFX ERR
			       " PHYPWR register is not set to power the Analog block power (%08lx)\n",
			       i);
			return 1;
		}

		if (i & S3C2443_PHYPWR_PLL_REF_CLK) {
			printk(KERN_INFO PFX ERR
			       " PHYPWR register is not configured to use the external crystal clock (%08lx)\n",
			       i);
			return 1;
		}

		i = __raw_readl(S3C2443_HCLKCON);	/* check the host clock */
		if (!(i & S3C2443_HCLKCON_USBHOST)) {
			printk(KERN_INFO PFX ERR
			       " HCLKCON is not configured to feed usbhost (%08lx)\n",
			       i);
			return 1;
		}

		i = __raw_readl(S3C2443_PHYCTRL);

		/* we use the a crystal */
		if ((i & S3C2443_PHYCTRL_CLK_SEL_MASK) !=
		    S3C2443_PHYCTRL_CLK_SEL_48MHZ) {
			printk(KERN_INFO PFX ERR
			       " PHYCTRL register is not set to use the 48Mhz clock (%08lx)\n",
			       i);
			return 1;
		}
		/* we use the crystal as clock source */
		if (i & S3C2443_PHYCTRL_EXT_CLK) {
			printk(KERN_INFO PFX ERR
			       " PHYCTRL register is not set to use the crystal as clock (%08lx)\n",
			       i);
			return 1;
		}

		//TODO!!!
		break;
	case GOCPU_S3C2412:
		printk(KERN_INFO PFX " %s setting clocks for usb host\n",
		       __FUNCTION__);
		/* tomtomgo_init configures the upll clock to 24 Mhz and
		   use a different divider in this code we reprogram the
		   clocks for the 4112. buspower.c also  changed the
		   frequency and settings of the usb host/device mode.
		   we have tried to keep the changes related to usb host to
		   this file, and thus we override both the buspower and
		   tomtomgo_init settings */

		/* we never use the EXTCLK but we use a crystal as source
		   clock that crystal is 12 Mhz */

		/* configure the clock source for the u(sb)pll */
		i = __raw_readl(S3C2412_CLKSRC);
		i &= ~(S3C2412_CLKSRC_SELUREF_MASK);	/* clean up the upll clock source mask( 3 << 12) */
		i |= S3C2412_CLKSRC_SELUREF_EXTOSC;	/* configure the external crystal to be clock source (2 << 12) */

		/* We now have a 12Mhz clock signal to the upll */
		/* Select the upll output to be the source for the USB system clock (USYSCLK = FOUTupll) */
		/* SELUPLL =1 */
		i |= S3C2412_CLKSRC_SELUPLL;	/* (1 <<5) */

		/* the USYSCLK on it's turn must be used a source for the USBSRCCLK */
		/* SELUSB =0 */
		i &= ~(S3C2412_CLKSRC_SELUSB);	/* 1 < 10 */

		__raw_writel(i, S3C2412_CLKSRC);

		/*configure the usb divider */
		i = __raw_readl(S3C2410_CLKDIVN);
		/* 1 = 48Mhz / 2 | 0 = 48Mhz */
		i &= ~(S3C2412_CLKDIVN_USB48DIV);
		__raw_writel(i, S3C2410_CLKDIVN);

		/* We now need to configure the upll and put it on fire */
		//FOUTupll 48Mhz p = 7 m = 64 (0x40) , s 1
		//[20] =0 (ON)
		//[19:12] = MDIV
		//[9:4] = PDIV
		//[0,1] = SDIV
		i &= ~(S3C2410_PLLCON_MDIVMASK | S3C2410_PLLCON_PDIVMASK |
		       S3C2410_PLLCON_SDIVMASK);
		i |= 0x40 << S3C2410_PLLCON_MDIVSHIFT;
		i |= 7 << S3C2410_PLLCON_PDIVSHIFT;
		i |= 1 << S3C2410_PLLCON_SDIVSHIFT;
		__raw_writel(i, S3C2410_UPLLCON);

		mdelay(1);

		/*enable the usb host */
		i = __raw_readl(S3C2410_CLKCON);
		i |= S3C2412_CLKCON_USBH;
		i |= S3C2412_CLKCON_USBH48M;
		/*dissable the usb device mode */
		i &= ~(S3C2412_CLKCON_USBD);
		i &= ~(S3C2412_CLKCON_USBD48M);

		__raw_writel(i, S3C2410_CLKCON);

		//                      IO_Deactivate(USB_PULL_EN);

		/* Select USBD instead of USBH1, enable USB port 1 */

	default:
		break;
	}

	return 0;		/* success */
};

/*
Called when we are entering the device mode detect.
I this mode we want to look if we are connected to a usb host like a computer.
to perform this task we must have two things in place
-1 The pull(up/down???) resistor must be set so that the computer will detect
that there is a device connected to the usb cable.
-2 We must listen to the usb host's message in order to detect it sends a reset signal.

If we get a signal within 2 seconds we can be sure that we are connected to a usb host.
We have finished the probe , we disable the pull(up/down???) as fast as possible so that
the usb host thinks we already disconnected. and send a message to user land telling
that we have detected that there is a usb host on the other side.

If we do no get a signal withing 2 seconds we know we are not connected to a usb-host.
we are probably connected to a CLA (car lighter adapter) and must possibly start behaving
like a usb host
*/
int device_detect_enter(struct usbmode_data *data)
{
	printk(KERN_DEBUG PFX " device_detect_enter\n");
	/* check data */
	if (data == NULL) {
		printk(KERN_DEBUG PFX ERR " %s  data == null \n", __FUNCTION__);
	}

	unsigned int test_value;
	unsigned int i;
	struct clk *clk;

	/* TODO: first configure clocks so interrupts etc can be triggered */
	if (setup_clocks_for_device(data)) {
		printk(KERN_DEBUG PFX ERR " Clock setup failed \n");
		return 1;
	}

	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:
		/*enable the pull-up so we are detected by the host is there is one */
		i = __raw_readl(S3C2443_UCLKCON);
		i |= S3C2443_UCLKCON_DETECT_VBUS;
		__raw_writel(i, S3C2443_UCLKCON);

		/* EP0-2 Interrupt enable */
		__raw_writel(S3C_HSUDC_INT_EP0 | S3C_HSUDC_INT_EP1 |
			     S3C_HSUDC_INT_EP2, S3C_HSUDC_EP_INT_EN_REG);

		__raw_writel(0x0000, S3C_HSUDC_TEST_REG);
		__raw_writel(S3C_HSUDC_INT_RESET | S3C_UDC_USBINT_SUSPEND,
			     S3C_HSUDC_SYS_STATUS_REG);
		test_value =
		    request_irq(IRQ_USBD, usb2_irq_handler, 0,
				"USB device detect IRQ driver II", data);
		if (test_value != 0) {
			printk(KERN_INFO PFX ERR
			       " failed to install irq (%d)\n", test_value);
			return 1;
		}
		break;
	default:
		/* Configure interrupts */
		IO_Activate(USB_PULL_EN);
		mdelay(100);

		/* disable EP0-4 USBD interrupts for now */
		/*kejo:2007-05-09 even if S3C_UDC_USB_INT_EN_REG I still get EP interrupts why? */

		__raw_writel(S3C_UDC_INT_EP0 | S3C_UDC_INT_EP1 |
			     S3C_UDC_INT_EP2, S3C_UDC_EP_INT_EN_REG);
		/* writing to the registers clears the data as stated in the documentation
		   we need to do that because we want to detect a reset signal. but (also in the
		   documentation ) when power (Vbus) is detected a reset signal is also generated
		   we do not want to react on that signal. This is also why we first clear the
		   registers and only after that request the interrupt. We first wait 100 ms to be sure
		   that the vbus power is stable, if this is not the case we could clear the register
		   and still get a reset interrupt because the state has been low */

		__raw_writel(0xff, S3C_UDC_USB_INT_REG);
		__raw_writel(0xff, S3C_UDC_EP_INT_REG);
		/* register an irq handler and use "data" as data_pointer */
		test_value =
		    request_irq(IRQ_USBD, device_detect_irq_handler, 0,
				"USB device detect IRQ driver", data);
		if (test_value != 0) {
			printk(KERN_INFO PFX ERR
			       " failed to install irq (%d)\n", test_value);
			return 1;
		}

	}
	/* get the usb-device clock and increase the use counter */
	clk = clk_get(NULL, "usb-device");
	if (IS_ERR(clk)) {
		printk(KERN_INFO PFX ERR " cannot get usb-device clock\n");
		goto err;
	}
	clk_use(clk);

	data->device_detect_timeout.expires =
	    jiffies + DEVICE_DETECT_TIMEOUT_SECONDS * HZ;
	data->device_detect_timeout.data = (unsigned long)data;
	data->device_detect_timeout.function = device_detect_timeout;
	add_timer(&data->device_detect_timeout);

	return 0;		/* Success */
      err:
	free_irq(IRQ_USBD, data);
	return 1;		/*failure */
}

/* Called when we are exiting the device mode detect (no matter what mode we wil be in next.
We try to stop anything that we started in device_detect_enter */
int device_detect_exit(struct usbmode_data *data)
{
	//disable_irq(IRQ_USBD);    
	struct clk *clk;
	unsigned int i;
	printk(KERN_DEBUG PFX " %s\n", __FUNCTION__);

	/* get the usb-device clock and decrease the use counter */
	clk = clk_get(NULL, "usb-device");
	if (IS_ERR(clk)) {
		printk(KERN_INFO PFX ERR " cannot get usb-device clock\n");
	} else {
		clk_unuse(clk);
	}
	/* remove the timeout timer so we will not get false positive
	   timeouts */
	del_timer_sync(&data->device_detect_timeout);

	free_irq(IRQ_USBD, data);

	switch (IO_GetCpuType()) {
	case GOCPU_S3C2443:
		i = __raw_readl(S3C2443_UCLKCON);
		i &= ~(S3C2443_UCLKCON_DETECT_VBUS);
		__raw_writel(i, S3C2443_UCLKCON);
		/* disable USB_PULL_EN */
		break;
	default:
		IO_Deactivate(USB_PULL_EN);
	}

	return 0;		/* success */
}

/*==end device detect mode **/

int initial_to_idle(struct usbmode_data *data)
{
	helper_disable_host_port(data);
	printk(KERN_DEBUG PFX " %s  \n", __FUNCTION__);
	unsigned long i;
	if (IO_GetCpuType() == GOCPU_S3C2443) {
		i = __raw_readl(S3C_HSUDC_SYS_CON_REG);

//        if (  i &  S3C_HSUDC_SYS_CON_VBUS_MASK ){
//            printk(KERN_DEBUG PFX ERR " VBUS on or off interrupt enabled %08lx\n",i);
//        }
		i = __raw_readl(S3C2443_UCLKCON);
		i |= S3C2443_UCLKCON_DETECT_VBUS;
		__raw_writel(i, S3C2443_UCLKCON);
	} else {
		/* 2410/2412/2440/2442 */
		i = __raw_readl(s3c24xx_misccr);

		/* disable port 0 (put it in suspend mode) and enable port 1 */
		i |= S3C2410_MISCCR_USBSUSPND0;
		i &= ~S3C2410_MISCCR_USBSUSPND1;

		/* There are two ports the s3c2440 documentation states
		   that that SEL_USBPAD (miscr[3]) selects port usb1 as host
		   or device, from that documentation is looks like we could
		   only enable usb1 as usb host. when this value is 0 this port is used
		   as device */
		i &= ~S3C2410_MISCCR_USBHOST;	/* make sure it's 0 to be in device mode */

		__raw_writel(i, s3c24xx_misccr);
	}
	data->usb_state = USB_STATE_IDLE;
	return 0;		/* success */
};

/* start state changes */
int idle_to_device_detect(struct usbmode_data *data)
{
	printk(KERN_DEBUG PFX " %s \n", __FUNCTION__);
	unsigned int i;

	switch (IO_GetCpuType()) {

	case GOCPU_S3C2443:
		/* enable the port set [12] to 0  */
		i = __raw_readl(s3c24xx_misccr);
		i &= ~S3C2443_MISCCR_USBSUSPND;	/* SEL_SUSPND in the documentation */
		__raw_writel(i, s3c24xx_misccr);

		break;
	default:
		/* check S3C2410_MISCCR register expectation */
		i = __raw_readl(s3c24xx_misccr);

		/* check the port 0 is disabled  {0 = normal mode 1= suspend mode } */
		if (!i & S3C2410_MISCCR_USBSUSPND0) {
			printk(KERN_DEBUG PFX ERR
			       " USB port 0 is not disabled \n");
			//TODO:return error
		}
		/* port 1 must be enabled */
		if (i & S3C2410_MISCCR_USBSUSPND1) {
			printk(KERN_DEBUG PFX ERR
			       " USB port 1 is not enabled \n");
		}
		/* we do no want the pad to be in usb host mode  0 = use USB1 as Device */
		if (!i & S3C2410_MISCCR_USBHOST) {
			printk(KERN_DEBUG PFX ERR
			       " USB port 1 set in host mode \n");
		}
		//printk(KERN_DEBUG PFX " VALUE %08x\n",i);
	}

	if (device_detect_enter(data)) {
		printk(KERN_DEBUG PFX ERR
		       " error in  %s  while switching from idle to device detect mode\n",
		       __FUNCTION__);
		return 1;
	}
	/* pull down enable */
	/* register usb device mode interrupt handler */

	//switch to detect mode??
	data->usb_state = USB_STATE_DEVICE_DETECT;
	printk(KERN_DEBUG PFX " %s SUCCESS\n", __FUNCTION__);
	return 0;		/* success */
};

int device_detect_to_idle(struct usbmode_data *data)
{
	printk(KERN_DEBUG PFX " %s \n", __FUNCTION__);
	if (device_detect_exit(data)) {
		printk(KERN_DEBUG PFX ERR
		       " %s error in  device_detect_exit while switching from device detect to idle mode\n",
		       __FUNCTION__);
		return 1;
	}

	data->usb_state = USB_STATE_IDLE;
	printk(KERN_DEBUG PFX " %s SUCCESS\n", __FUNCTION__);
	return 0;		/* success */
};

/* Switch to device mode. This mode is not implemented
in the current kernel, this does via the bootloader.
The only thing we really want to do in this mode is to disable
the pull_enable so the connected host does not try to assign a
usb address to the device.Why? because we will reboot and only after that
be ready to behave like a usb device */
int device_detect_to_device(struct usbmode_data *data)
{
	//exit device detect mode...
	if (device_detect_exit(data)) {
		printk(KERN_DEBUG PFX ERR
		       " %s error in  device_detect_exit while switching from device detect to device mode\n",
		       __FUNCTION__);
		return 1;
	}

	data->usb_state = USB_STATE_DEVICE;
	printk(KERN_DEBUG PFX " %s SUCCESS\n", __FUNCTION__);
	return 0;		/* success */

};

int device_to_idle(struct usbmode_data *data)
{
	data->usb_state = USB_STATE_IDLE;
	printk(KERN_DEBUG PFX " %s SUCCESS\n", __FUNCTION__);
	return 0;		/* success */
};

int usb_count_devs(struct usb_device *usbdev)
{
	int count = 0;
	int loop = 0;

	count++;
	if (usbdev->maxchild != 0) {
		for (loop = 0; loop < usbdev->maxchild; loop++)
			if (usbdev->children[loop] != NULL)
				count += usb_count_devs(usbdev->children[loop]);
	}
	return count;
}

/**
 * return the amount of connected devices connected to the root hub in use for OTG (not counting the internal USB port)
 * Note: We ignore the roothub port 0, as this contains the bluetooth device that'll always be there (or not)
 **/
int usb_get_connected_device_count(void)
{
	struct usb_device *roothub;
	unsigned long int shiftval;
	int count = 0;
	int devcount = 0;

	roothub = usb_find_device(0, 0);
	shiftval = IO_UsbOhciPortMask();

	/* Find out which devices are in use on the roothub. IO_UsbOhciPortMask is a bitmask which has all ports */
	/* that are not in use set to 1. */
	if (roothub != NULL) {
		/* Cycle through all ports on the roothub, but exclude any port that is marked as not in use */
		/* These busses don't count for this code, but might contain a bluetooth chip. This is */
		/* irrelevant to this code however. */
		for (count = 0; count < roothub->maxchild; count++)
			if (!(shiftval & (1 << count))
			    && (roothub->children[count] != NULL))
				devcount +=
				    usb_count_devs(roothub->children[count]);
	}
	return devcount;
}

void host_device_removal_watcher(void *data_pointer)
{
	struct usbmode_data *data = (struct usbmode_data *)data_pointer;
	if (usb_get_connected_device_count() == 0) {
		printk(KERN_DEBUG PFX
		       " %s no device connected any more, disconnecting\n",
		       __FUNCTION__);
		data->input_state |= INPUT_HOST_DEVICE_DISCONNECT;
		schedule_work(&data->state_changed_work);
		return;
	};

	schedule_delayed_work(&data->host_device_removal_watcher_timeout,
			      HOST_DEVICE_REMOVAL_WATCHER_TIMEOUT_SECONDS * HZ);
}

void host_detect_timeout(void *data_pointer)
{
	printk(KERN_DEBUG PFX " %s \n", __FUNCTION__);
	struct usbmode_data *data = (struct usbmode_data *)data_pointer;
	if (data->usb_state == USB_STATE_HOST_DETECT) {
		struct bus_type *usb_bus = find_bus("usb");
		if (!usb_bus) {
			printk(KERN_INFO PFX
			       " failed to get the usb bus. Is usb compiled into the kernel?");
			/* In this situation we don't have usb-host support. just send a timeout event */
			if (!(data->input_state & INPUT_HOST_DETECT_TIMEOUT)) {
				printk(KERN_DEBUG PFX
				       " %s no-host... sending timeout\n",
				       __FUNCTION__);
				data->input_state |= INPUT_HOST_DETECT_TIMEOUT;
				schedule_work(&data->state_changed_work);
			} else {
				printk(KERN_DEBUG PFX ERR
				       " %s INPUT_HOST_DETECT_TIMEOUT already set in no-host mode\n",
				       __FUNCTION__);
			}
			return;
		} else if (usb_get_connected_device_count() > 0) {
			/* In this situation we have a device connected to the pnd */
			if (!(data->input_state & INPUT_HOST_DEVICE_CONNECT)) {
				data->input_state |= INPUT_HOST_DEVICE_CONNECT;
				schedule_work(&data->state_changed_work);
			} else {
				printk(KERN_DEBUG PFX ERR
				       " %s INPUT_HOST_DEVICE_CONNECT already set %02x %i %i %i\n",
				       __FUNCTION__, data->input_state,
				       data->input_state,
				       INPUT_HOST_DEVICE_CONNECT,
				       data->
				       input_state & INPUT_HOST_DEVICE_CONNECT);
			}
		} else {
			/* There is a usb_bus but device is connected to the bus.
			   this just means that no device was detected */
			if (!(data->input_state & INPUT_HOST_DETECT_TIMEOUT)) {
				printk(KERN_DEBUG PFX " %s timeout\n",
				       __FUNCTION__);
				data->input_state |= INPUT_HOST_DETECT_TIMEOUT;
				schedule_work(&data->state_changed_work);
			} else {
				printk(KERN_DEBUG PFX ERR
				       " %s INPUT_HOST_DETECT_TIMEOUT already set\n",
				       __FUNCTION__);
			}
		}
	} else {
		/* we are not in HOST_DETECT mode but still arrive here.
		   when can that happen?. One situation where this happens
		   is when just before the timeout occurs you remove the
		   power from the bus, in that situation it can happen
		   that while you are not in HOST_DETECT mode you still can 
		   receive this event. The sate machine can handle any event in 
		   any situation but we will for the moment not generate the event
		   since we simply are not in HOST_DETECT mode. Still lets write a
		   message because the biggest chance of reaching this code is actually 
		   that somebody forgot to disable the host_detect timeout while switching
		   to a certain mode */
		printk(KERN_DEBUG PFX
		       " %s timeout while not in HOST_DETECT mode\n",
		       __FUNCTION__);
	}
}

int device_detect_to_host_detect(struct usbmode_data *data)
{

	/* First exit device detect mode */
	if (device_detect_exit(data)) {
		printk(KERN_DEBUG PFX ERR
		       " %s error in  device_detect_exit while switching from device detect to host detect mode\n",
		       __FUNCTION__);
		return 1;
	}

	if (setup_clocks_for_host(data)) {
		printk(KERN_DEBUG PFX ERR
		       " %s failed to setup usb host but ignoring for the moment..\n",
		       __FUNCTION__);
		//        printk(KERN_WARN PFX ERR " %s error in  setup_clocks_for_host while switching from device detect to host detect mode\n",__FUNCTION__);
		//return 1;

	};
	helper_enable_host_port(data);

	schedule_delayed_work(&data->host_detect_timeout,
			      HOST_DETECT_TIMEOUT_SECONDS * HZ);

	data->usb_state = USB_STATE_HOST_DETECT;
	return 0;		/* success */

};

int host_detect_to_idle(struct usbmode_data *data)
{
	helper_disable_host_port(data);
	cancel_delayed_work(&data->host_detect_timeout);
	cancel_delayed_work(&data->host_device_removal_watcher_timeout);
	data->usb_state = USB_STATE_IDLE;
	return 0;		/* success */
}

int host_detect_to_cla(struct usbmode_data *data)
{
	/*disable the detect workqueue */
	cancel_delayed_work(&data->host_device_removal_watcher_timeout);
	data->usb_state = USB_STATE_CLA;
	return 0;		/* success */
};

int host_detect_to_host(struct usbmode_data *data)
{

	/*disable the detect timer(in this case we don't expect the timer to be running */
	/* this causes this kind of "error" messages
	   one|2007-07-05 11:20:21| Uninitialised timer!
	   one|2007-07-05 11:20:21| This is just a warning.  Your computer is OK
	   one|2007-07-05 11:20:21| function=0x00000000, data=0x0
	 */

	cancel_delayed_work(&data->host_device_removal_watcher_timeout);

	/* Schedule delayed work */
	schedule_delayed_work(&data->host_device_removal_watcher_timeout,
			      HOST_DEVICE_REMOVAL_WATCHER_TIMEOUT_SECONDS * HZ);

	data->usb_state = USB_STATE_HOST;
	return 0;		/* success */
};

int cla_to_idle(struct usbmode_data *data)
{
	helper_disable_host_port(data);
	data->usb_state = USB_STATE_IDLE;
	return 0;
};

int host_to_idle(struct usbmode_data *data)
{
	printk(KERN_DEBUG PFX " %s DISABLING PORT\n", __FUNCTION__);
	cancel_delayed_work(&data->host_device_removal_watcher_timeout);
	helper_disable_host_port(data);
	data->usb_state = USB_STATE_IDLE;
	return 0;
};
