/* drivers/video/tomtom/lta058b3l0f.c
*
* Specific functionality for the Toshiba LTA058B3L0F screen.
*
* Copyright (C) 2008 TomTom BV <http://www.tomtom.com/>
* Author: Peter Wouters <peter.wouters@tomtom.com>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*/

#ifdef __KERNEL__
#include <linux/delay.h>
#include <linux/cpufreq.h>
#include <barcelona/gopins.h>
#include <linux/err.h>
#include <asm/hardware/clock.h>
#endif /* __KERNEL */
#ifdef __BOOTLOADER__
#include "timer.h"
#include "gopins.h"
#endif /* __BOOTLOADER__ */
#include "lcd_controller.h"
#include "screeninfo.h"
#include "framerate.h"
#include "lta058b3l0f.h"
#include "lcdregs.h"

// LCDCON1
#define MMODE		0
#define PNR_MODE	3
#define BPP_MODE	16
#define ENVID		0

// LCDCON2
#define VBPD		1
#define VFPD		20
#define VSPW		9

// LCDCON3
#define HBPD		0
#define HFPD		5

// LCDCON4

#define MVAL      13
#define HSPW      41

// LCDCON5
#define BPP24BL   0
#define FRM565    1
#define INVVCLK   1
#define INVVLINE  1
#define INVVFRAME 1
#define INVVD     0
#define INVVDEN   0
#define INVPWREN  0
#define INVLEND   1
#define PWREN     1
#define ENLEND    0
#define BSWP      0
#define HWSWP     1



static struct lcd_additional_info add_lcdspecs = { 
   .bswp = BSWP, .hwswp = HWSWP, .invvclk = INVVCLK, .invvline = INVVLINE, .invvframe = INVVFRAME, .invvd = INVVD, .invvden = INVVDEN,
   .pwren = PWREN, .invpwren = INVPWREN, .enlend = ENLEND, .invlend = INVLEND,
   .envid = ENVID, .mmode = MMODE, .mval = MVAL, .bpp24bl = BPP24BL, .frm565 = FRM565,
   .hozval = 0, .lineval = 0, .divider = 0 
};

void LTA058B3L0F_Init(void)
{

   add_lcdspecs.divider= 6;		// 10 MHz clock for Toshiba display

   get_hozlineval( &ttgfb_lta058b3l0f_lcdspecs, &add_lcdspecs.hozval, &add_lcdspecs.lineval );

   ttgfb_lcd_controller->initialize(&ttgfb_lta058b3l0f_lcdspecs, &add_lcdspecs);

   mdelay(1);  /* Must wait at least 0.5 ms after Sync is enabled before the DISP_ON is enabled */
  
   /* According to display spec, the DISP_ON (=!LCD_RESET) should not be enabled during VSYNC.
      So wait until Vertical Status is ACTIVE, so we have at least the front-porch time to 
      switch DISP_ON on */
   if ((rVIDCON0 & 0x00000003) != 0)  /* Check if TFT controller is enabled */
   {
     while ((rVIDCON1 & 0x0006000) != 0x00004000) /* Wait until Vertical status is ACTIVE */
    {
    }
  }
   IO_Deactivate(LCD_RESET); /* Set DISP_ON */
}

void LTA058B3L0F_Off(void)
{
   IO_Activate(LCD_RESET);
   mdelay(100);

   ttgfb_lcd_controller->stop_lcd();
}

#ifdef CONFIG_CPU_FREQ
struct lcd_screen_info ttgfb_lta058b3l0f_lcdspecs = {"TOS", "LTA058B3L0F", 95, 54, 480, 272, 25, 75, LTA058B3L0F_Init, LTA058B3L0F_Off, NULL,
   PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}},
   lcd_def_freq_policy, lcd_def_freq_trans};
#else
struct lcd_screen_info ttgfb_lta058b3l0f_lcdspecs = {"TOS", "LTA058B3L0F", 95, 54, 480, 272, 25, 75, LTA058B3L0F_Init, LTA058B3L0F_Off, NULL,
   PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}}};
#endif


