/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of SUNY
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "union.h"

/* compatibility code between 2.6.22 and 2.6.9 code */


/*
 * kstrdup - allocate space for and copy an existing string
 *
 * @s: the string to duplicate
 * @gfp: the GFP mask used in the kmalloc() call when allocating memory
 */
/*removed for 2.6.13 char *kstrdup(const char *s, int gfp)
{
	size_t len;
	char *buf;

	if (!s)
		return NULL;

	len = strlen(s) + 1;
	buf = kmalloc(len, gfp);
	if (buf)
		memcpy(buf, s, len);
	return buf;
}*/

/* inefficient version of krealloc: allways free's and reallocates */
void *krealloc2(size_t oldsize, void *ptr, size_t newsize, int flags)
{
  void *newptr;			/* pointer of new buffer */
  int numtocopy;		/* number of bytes copied into new buffer */

  /*
   * optimization (OPTIONAL): if oldsize is same as newsize, and ptr is not
   * NULL, then there's not need to realloc/malloc.
   */
  if (oldsize == newsize  &&  ptr)
	  return ptr;

  /* allocate space for newptr */
  newptr = (void *) kmalloc(newsize, flags);
  if (!newptr)
	  return NULL;		/* malloc failed */

  /* check if this is a new allocation */
  if (!ptr)
	  return newptr;

  /* figure out how many bytes we need to copy */
  /* can be done with MIN(oldsize,newsize) */
  numtocopy = (newsize < oldsize ? newsize : oldsize);

  /* now copy a number of bytes into new space */
  memcpy(newptr, ptr, numtocopy);

  /* free old pointer */
  kfree(ptr);

  /* return result */
  return newptr;
}

