/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of SUNY
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef _UNIONFS_COMPAT_H_
#define _UNIONFS_COMPAT_H_

/* compatibility code between 2.6.22 and 2.6.9 code */

/*Removed for 2.6.13 extern char *kstrdup(const char *s, int gfp);*/
extern void *krealloc2(size_t oldsize, void *ptr, size_t newsize, int flags);

#define mutex			semaphore
#define mutex_init(x)		init_MUTEX((x))
#define mutex_lock(x)		down((x))
#define mutex_unlock(x)		up((x))
#define mutex_is_locked(x)	(1) /* XXX: ugly */
#define	i_mutex			i_sem
#define kzalloc(num, flags)	kcalloc(1, (num), (flags))
#define UNIONFS_SUPER_MAGIC	0xf15f083d
#define	POISON_INUSE	0x5a	/* for use-uninitialised poisoning */
#define SEEK_SET	0	/* seek relative to beginning of file */
#define SEEK_CUR	1	/* seek relative to current file position */
#define SEEK_END	2	/* seek relative to end of file */
#define SEEK_MAX	SEEK_END
#define lookup_one_len_nd(name, base, len, nd)	\
		lookup_one_len((name), (base), (len))
/* define compatible boolean type */
#define bool int
#define false (0)
#define true (1)

#define gfp_t			int
#define AOP_WRITEPAGE_ACTIVATE	WRITEPAGE_ACTIVATE

/* no lockdep support, but keep hooks */
#define lockdep_off()
#define lockdep_on()
#define mutex_lock_nested(x,y)	mutex_lock((x))
#define down_read_nested(x,y)	down_read((x))
#define I_MUTEX_PARENT

struct path {
	struct vfsmount *mnt;
	struct dentry *dentry;
};

static inline void pathget(struct path *path)
{
	mntget(path->mnt);
	dget(path->dentry);
}

static inline void pathput(struct path *path)
{
	dput(path->dentry);
	mntput(path->mnt);
}

/*
 * lhs < rhs:  return <0
 * lhs == rhs: return 0
 * lhs > rhs:  return >0
 */
static inline int timespec_compare(const struct timespec *lhs,
				   const struct timespec *rhs)
{
	if (lhs->tv_sec < rhs->tv_sec)
		return -1;
	if (lhs->tv_sec > rhs->tv_sec)
		return 1;
	return lhs->tv_nsec - rhs->tv_nsec;
}

/* cheasy compatibility function, instead of hacking it at every arch */
/* if you don't like it, it's time to move up to 2.6.22 or newer. :-) */
/*Removed for 2.6.13 static inline int atomic_inc_return(atomic_t *v)
{
  atomic_inc(v);
  return atomic_read(v);
}*/

/*
 * Return byte-offset into filesystem object for page.
 */
/*Removed for 2.6.13 static inline loff_t page_offset(struct page *page)
{
	return ((loff_t)page->index) << PAGE_CACHE_SHIFT;
}*/

/* from include/linux/fs.h on 2.6.18+ */
static inline void drop_nlink(struct inode *inode)
{
	inode->i_nlink--;
}
static inline void inode_dec_link_count(struct inode *inode)
{
	drop_nlink(inode);
	mark_inode_dirty(inode);
}
#endif /* _UNIONFS_COMPAT_H_ */
