/* linux/include/asm/arch-s3c2450/regs-iis.h
 *
 * S3C2450 IIS register definitions
 *
 * Copyright (C) 2009 TomTom BV <http://www.tomtom.com/>
 * Author: Robert Delien <robert.delien@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *      Changelog:
 *      29-05-2009      rodl    First creation
 */

#ifndef __ASM_ARCH_S2C2450_REGS_IIS_H
#define __ASM_ARCH_S2C2450_REGS_IIS_H

/*
 * Base addresses
 */
#define S2C2450_IIS0_BASE		(0x55000100)	/* IIS bus interface */
#define S2C2450_IIS1_BASE		(0x55000000)	/* IIS multi audio interface */

/*
 * IISCON Register
 */
#define S3C2450_IISCON			(0x00000000)	/* Reset value 0x00000600 */
							/* Bits [31..18] Reserved, program to zero */
#define S3C2450_IISCON_FTXURSTATUS	(1<<17)		/* R/W */
#define S3C2450_IISCON_FTXURINTEN	(1<<16)		/* R/W */
#define S3C2450_IISCON_FTX2EMPT		(1<<15)		/* R (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISCON_FTX1EMPT		(1<<14)		/* R (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISCON_FTX2FULL		(1<<13)		/* R (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISCON_FTX1FULL		(1<<12)		/* R (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISCON_LRI		(1<<11)		/* R */
#define S3C2450_IISCON_FTX0EMPT		(1<<10)		/* R */
#define S3C2450_IISCON_FRXEMPT		(1<<9)		/* R */
#define S3C2450_IISCON_FTX0FULL		(1<<8)		/* R */
#define S3C2450_IISCON_FRXFULL		(1<<7)		/* R */
#define S3C2450_IISCON_TXDMAPAUSE	(1<<6)		/* R/W */
#define S3C2450_IISCON_RXDMAPAUSE	(1<<5)		/* R/W */
#define S3C2450_IISCON_TXCHPAUSE	(1<<4)		/* R/W */
#define S3C2450_IISCON_RXCHPAUSE	(1<<3)		/* R/W */
#define S3C2450_IISCON_TXDMACTIVE	(1<<2)		/* R/W */
#define S3C2450_IISCON_RXDMACTIVE	(1<<1)		/* R/W */
#define S3C2450_IISCON_I2SACTIVE	(1<<0)		/* R/W */

/*
 * IISMOD Register
 */
#define S3C2450_IISMOD			(0x00000004)	/* Reset value 0x00000000 */
							/* Bits [31..22] Reserved, program to zero */
#define S3C2450_IISMOD_CDD2MASK		(3<<20)		/* Mask              (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD2NONE		(0<<20)		/* R/W No Discard    (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD2LOW		(1<<20)		/* R/W 15:0 Discard  (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD2HIGH		(2<<20)		/* R/W 31:16 Discard (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD2RES		(3<<20)		/* R/W Reserved      (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD1MASK		(3<<18)		/* Mask              (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD1NONE		(0<<20)		/* R/W No Discard    (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD1LOW		(1<<20)		/* R/W 15:0 Discard  (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD1HIGH		(2<<20)		/* R/W 31:16 Discard (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_CDD1RES		(3<<20)		/* R/W Reserved      (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_DCE2		(1<<17)		/* R/W (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISMOD_DCE1		(1<<16)		/* R/W (IIS1 only! Reserved on IIS0, program to zero) */
							/* Bit [15] Reserved, program to zero */
#define S3C2450_IISMOD_BLCMASK		(3<<13)		/* Mask */
#define S3C2450_IISMOD_BLC16BITS	(0<<13)		/* R/W 16 Bits per channel */
#define S3C2450_IISMOD_BLC8BITS		(1<<13)		/* R/W  8 Bits per channel */
#define S3C2450_IISMOD_BLC24BITS	(2<<13)		/* R/W 24 Bits per channel */
#define S3C2450_IISMOD_BLCRES		(3<<13)		/* R/W Reserved */
#define S3C2450_IISMOD_CDCLKCON		(1<<12)		/* R/W */
#define S3C2450_IISMOD_IMSMASK		(3<<10)		/* Mask */
#define S3C2450_IISMOD_IMSMASTERPCLK	(0<<10)		/* R/W Master mode, PCLK is source */
#define S3C2450_IISMOD_IMSMASTERCLKAUDIO (1<<10)	/* R/W Master mode, CLKAUDIO is source */
#define S3C2450_IISMOD_IMSSLAVEPCLK	(2<<10)		/* R/W Slave mode, PCLK is source */
#define S3C2450_IISMOD_IMSSLAVECLKAUDIO	(3<<10)		/* R/W Slave mode, CLKAUDIO is source */
#define S3C2450_IISMOD_TXRMASK		(3<<8)		/* Mask */
#define S3C2450_IISMOD_TXRTXONLY	(0<<8)		/* R/W Transmit only mode */
#define S3C2450_IISMOD_TXRRXONLY	(1<<8)		/* R/W Receive only mode */
#define S3C2450_IISMOD_TXRTRXSIM	(2<<8)		/* R/W Transmit and receive simultaneous mode */
#define S3C2450_IISMOD_TXRRES		(3<<8)		/* R/W Reserved */
#define S3C2450_IISMOD_LRP		(1<<7)		/* R/W */
#define S3C2450_IISMOD_SDFMASK		(3<<5)		/* Mask */
#define S3C2450_IISMOD_SDFI2S		(0<<5)		/* R/W IIS format */
#define S3C2450_IISMOD_SDFMSB		(1<<5)		/* R/W MSB-justified format */
#define S3C2450_IISMOD_SDFLSB		(2<<5)		/* R/W LSB-justified format */
#define S3C2450_IISMOD_SDFRES		(3<<5)		/* R/W Reserved */
#define S3C2450_IISMOD_RFSMASK		(3<<3)		/* Mask */
#define S3C2450_IISMOD_RFS256FS		(0<<3)		/* R/W 256*fs */
#define S3C2450_IISMOD_RFS512FS		(1<<3)		/* R/W 512*fs */
#define S3C2450_IISMOD_RFS384FS		(2<<3)		/* R/W 384*fs */
#define S3C2450_IISMOD_RFS768FS		(3<<3)		/* R/W 512*fs */
#define S3C2450_IISMOD_BFSMASK		(3<<1)		/* Mask */
#define S3C2450_IISMOD_BFS32FS		(0<<1)		/* R/W 32*fs */
#define S3C2450_IISMOD_BFS48FS		(1<<1)		/* R/W 48*fs */
#define S3C2450_IISMOD_BFS16FS		(2<<1)		/* R/W 16*fs */
#define S3C2450_IISMOD_BFS24FS		(3<<1)		/* R/W 24*fs */
							/* Bit [0] Reserved, program to zero */

/*
 * IISFIC Register
 */
#define S3C2450_IISFIC			(0x00000008)	/* Reset value 0x00000000 */
											/* Bits [31..29] Reserved, program to zero */
#define S3C2450_IISFIC_FTXCNT2MASK	(0x1F<<24)					/* Mask (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISFIC_FTXCNT2(n)	(((n)&(S3C2450_IISFIC_FTXCNT2MASK))>>24)	/* R    (IIS1 only! Reserved on IIS0, program to zero) */
											/* Bits [23..21] Reserved, program to zero */
#define S3C2450_IISFIC_FTXCNT1MASK	(0x1F<<16)					/* Mask (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISFIC_FTXCNT1(n)	(((n)&(S3C2450_IISFIC_FTXCNT1MASK))>>16)	/* R    (IIS1 only! Reserved on IIS0, program to zero) */
#define S3C2450_IISFIC_TFLUSH		(1<<15)						/* R/W */
											/* Bits [14..13] Reserved, program to zero */
#define S3C2450_IISFIC_FTXCNT0MASK	(0x1F<<8)					/* Mask */
#define S3C2450_IISFIC_FTXCNT0(n)	(((n)&(S3C2450_IISFIC_FTXCNTMASK))>>8)		/* R */
#define S3C2450_IISFIC_RFLUSH		(1<<7)						/* R/W */
											/* Bits [6..5] Reserved, program to zero */
#define S3C2450_IISFIC_FRXCNTMASK	(0x1F<<0)					/* Mask */
#define S3C2450_IISFIC_FRXCNT(n)	(((n)&(S3C2450_IISFIC_FRXCNTMASK))>>0)		/* R */

/*
 * IISPSR Register
 */
#define S3C2450_IISPSR			(0x0000000C)	/* Reset value 0x00000000 */
							/* Bits [31..16] Reserved, program to zero */
#define S3C2450_IISPSR_PSRAEN		(1<<15)		/* R/W */
							/* Bit [14] Reserved, program to zero */
#define S3C2450_IISPSR_PSVALAMASK	(0x3F<<8)	/* Mask */
#define S3C2450_IISPSR_PSVALA(n)	(0x3F<<8)	/* R/W */
							/* Bits [7..0] Reserved, program to zero */

/*
 * IISTXD Register
 */
#define S3C2450_IISTXD			(0x00000010)	/* Reset value 0x00000000 */
#define S3C2450_IISTXD_R24MASK		(0xFFFFFF<<0)				/* Mask */
#define S3C2450_IISTXD_R24(n)		((n)&(S3C2450_IISTXD_R24MASK))		/* W */
#define S3C2450_IISTXD_L24MASK		(0xFFFFFF<<0)				/* Mask */
#define S3C2450_IISTXD_L24(n)		((n)&(S3C2450_IISTXD_L24MASK))		/* W */
#define S3C2450_IISTXD_R16MASK		(0xFFFF<<16)				/* Mask */
#define S3C2450_IISTXD_R16(n)		(((n)<<16)&(S3C2450_IISTXD_R16MASK))	/* W */
#define S3C2450_IISTXD_L16MASK		(0xFFFF<<0)				/* Mask */
#define S3C2450_IISTXD_L16(n)		(((n)<<0)&(S3C2450_IISTXD_L16MASK))	/* W */
#define S3C2450_IISTXD_R8MASK		(0xFF<<16)				/* Mask */
#define S3C2450_IISTXD_R8(n)		(((n)<<16)&(S3C2450_IISTXD_R8MASK))	/* W */
#define S3C2450_IISTXD_L8MASK		(0xFF<<0)				/* Mask */
#define S3C2450_IISTXD_L8(n)		(((n)<<0)&(S3C2450_IISTXD_L8MASK))	/* W */

/*
 * IISRXD Register
 */
#define S3C2450_IISRXD			(0x00000014)	/* Reset value 0x00000000 */
#define S3C2450_IISRXD_R24MASK		(0xFFFFFF<<0)				/* Mask */
#define S3C2450_IISRXD_R24(n)		(((n)&(S3C2450_IISRXD_R24MASK))>>0)	/* R */
#define S3C2450_IISRXD_L24MASK		(0xFFFFFF<<0)				/* Mask */
#define S3C2450_IISRXD_L24(n)		(((n)&(S3C2450_IISRXD_L24MASK))>>0)	/* R */
#define S3C2450_IISRXD_R16MASK		(0xFFFF<<16)				/* Mask */
#define S3C2450_IISRXD_R16(n)		(((n)&(S3C2450_IISRXD_R16MASK))>>16)	/* R */
#define S3C2450_IISRXD_L16MASK		(0xFFFF<<0)				/* Mask */
#define S3C2450_IISRXD_L16(n)		(((n)&(S3C2450_IISRXD_L16MASK))>>0)	/* R */
#define S3C2450_IISRXD_R8MASK		(0xFF<<16)				/* Mask */
#define S3C2450_IISRXD_R8(n)		(((n)&(S3C2450_IISRXD_R8MASK))>>16)	/* R */
#define S3C2450_IISRXD_L8MASK		(0xFF<<0)				/* Mask */
#define S3C2450_IISRXD_L8(n)		(((n)&(S3C2450_IISRXD_L8MASK))>>0)	/* R */

#endif /* __ASM_ARCH_S2C2450_REGS_IIS_H */
