/*
 * Camera driver for s3c241x camera interface
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * Author:      Ithamar R. Adema, <ithamar.adema@tomtom.com>
 */

#ifndef __INCLUDE_MEDIA_S3C_CAMIF_H
#define __INCLUDE_MEDIA_S3C_CAMIF_H

#include <linux/videodev2.h>

/*
 * These IOCTLs allow the default viewing window (see chapter 1.10 cropping and scaling) to be set.
 * Default V4L2 expects this to be a fixed number from which all cropping/scaling is done.
 * Note that even though the parameter is a struct v4l2_rect, the top and left fields should ALWAYS
 * be 0.
 */
#define VIDIOC_PRIV_SETDEFRES	_IOWR  ('V', 192, struct v4l2_rect)
#define VIDIOC_PRIV_GETDEFRES	_IOR  ('V', 193, struct v4l2_rect)

/* sleep/wake/suspend/resume ioctls */
#define VIDIOC_PRIV_SLEEP	_IO  ('V', 194)
#define VIDIOC_PRIV_WAKE	_IO  ('V', 195)
#define VIDIOC_PRIV_SUSPEND	_IO  ('V', 196)
#define VIDIOC_PRIV_RESUME	_IO  ('V', 197)

/* debug ioctls */
#define VIDIOC_PRIV_DUMPREGS	_IO  ('V', 198)

/* aspect ratio */
#define VIDIOC_PRIV_GET_WSS_DATA          _IOR ('V', 199, int)
#define VIDIOC_PRIV_SET_ASPECT_RATIO      _IOWR('V', 200, int)
#define VIDIOC_PRIV_GET_ASPECT_RATIO      _IOR ('V', 201, int)

/* input-output sync control*/
#define VIDIOC_PRIV_SYNC	 _IO  ('V', 202)

/* turn DMA on/off */
#define VIDIOC_PRIV_START_VIDEO	_IO  ('V', 203)
#define VIDIOC_PRIV_STOP_VIDEO	_IO  ('V', 204)

#undef USE_WITH_VDTEST
/*
If you want to use vdtest enable the USE_WITH_VDTEST define.
This will have effect in the following 3 files:
- linux-s3c24xx/drivers/media/video/s3c-camif/adv7180.c
- linux-s3c24xx/drivers/media/video/s3c-camif/cam-v4l.c
- linux-s3c24xx/include/media/s3c-camif.h
*/

#ifdef USE_WITH_VDTEST
#warning "For release builds undefine USE_WITH_VDTEST in linux-s3c24xx/include/media/s3c-camif.h\n"

#define VIDIOC_PRIV_CR	 	_IOR  ('V', 248, int)
#define VIDIOC_PRIV_CW	 	_IOWR ('V', 249, int)
#define VIDIOC_PRIV_CRR	 	_IOR  ('V', 250, int)
#define VIDIOC_PRIV_CRW	 	_IOWR ('V', 251, int)

#endif


/* screen ratios */
#define V4L2_PRIV_ASPECT_RATIO_FILL      0   /* fill the entire screen */
#define V4L2_PRIV_ASPECT_RATIO_AUTO      1   /* use WSS (not implemented, yet)  */
#define V4L2_PRIV_ASPECT_RATIO_4_3       2   /* force a 4:3 screen */
#define V4L2_PRIV_ASPECT_RATIO_14_9      3   /* force a 14:9 screen */
#define V4L2_PRIV_ASPECT_RATIO_16_9      4   /* force a 16:9 screen */
#define V4L2_PRIV_ASPECT_RATIO_4_3_ZOOM  5   /* force a 4:3 screen */
#define V4L2_PRIV_ASPECT_RATIO_14_9_ZOOM 6   /* force a 14:9 screen */
#define V4L2_PRIV_ASPECT_RATIO_16_9_ZOOM 7   /* force a 16:9 screen */
#endif
